#!/usr/bin/python
# Copyright (C) International Business Machines Corp., 2005
# Author: Josh Triplett <josh@freedesktop.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; under version 2 of the License.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
from xenstat import *
from SimpleXMLRPCServer import SimpleXMLRPCServer
from socket import gethostname
from time import time

def get_timestamp():
    return long(time()*1000000000)

def make_vcpu_struct(vcpu_id, vcpu):
    return {"id": vcpu_id,
            "ns": str(xenstat_vcpu_ns(vcpu))}

def make_network_struct(network):
    return {"id":       xenstat_network_id(network),
            "rbytes":   xenstat_network_rbytes(network),
            "rpackets": xenstat_network_rpackets(network),
            "rerrs":    xenstat_network_rerrs(network),
            "rdrop":    xenstat_network_rdrop(network),
            "tbytes":   xenstat_network_tbytes(network),
            "tpackets": xenstat_network_tpackets(network),
            "terrs":    xenstat_network_terrs(network),
            "tdrop":    xenstat_network_tdrop(network)}

def make_state_struct(domain):
    return {"dying":    xenstat_domain_dying(domain),
            "crashed":  xenstat_domain_crashed(domain),
            "shutdown": xenstat_domain_shutdown(domain),
            "paused":   xenstat_domain_paused(domain),
            "blocked":  xenstat_domain_blocked(domain),
            "running":  xenstat_domain_running(domain)}

def make_domain_struct(domain):
    num_vcpus = xenstat_domain_num_vcpus(domain)
    num_networks = xenstat_domain_num_networks(domain)
    return {
        "id":           xenstat_domain_id(domain),
        "state":        make_state_struct(domain),
        "resources": {
            "cpu_ns":    str(xenstat_domain_cpu_ns(domain)),
            "num_vcpus": num_vcpus,
            "vcpus": [make_vcpu_struct(vcpu, xenstat_domain_vcpu(domain, vcpu))
                      for vcpu in range(num_vcpus)],
            "cur_mem":      str(xenstat_domain_cur_mem(domain)),
            "max_mem":      str(xenstat_domain_cur_mem(domain)),
            "ssid":         xenstat_domain_ssid(domain),
            "num_networks": num_networks,
            "networks":
                [make_network_struct(xenstat_domain_network(domain, net))
                 for net in range(num_networks)]
            }
        }

def make_node_struct(node):
    num_domains = xenstat_node_num_domains(node)
    return {
        "timestamp":   str(get_timestamp()),
        "hostname":    gethostname(),
        "machine_stats": {
            "num_cpus":    xenstat_node_num_cpus(node),
            "cpu_hz":      str(xenstat_node_cpu_hz(node)),
            "tot_mem":     str(xenstat_node_tot_mem(node)),
            "free_mem":    str(xenstat_node_free_mem(node)),
            "num_domains": num_domains,
        },
        "domains": [make_domain_struct(xenstat_node_domain_by_index(node, i))
                    for i in range(num_domains)]
        }

def xenstat_gather():
    global handle
    node = xenstat_get_node(handle, XENSTAT_ALL)
    try:
        result = make_node_struct(node)
    finally:
        xenstat_free_node(node)
    return result

def main(args):
    global handle
    try:
        handle = xenstat_init()
        server = SimpleXMLRPCServer(("0.0.0.0", 8000))
        server.register_introspection_functions()
        server.register_function(xenstat_gather)
        server.serve_forever()
    finally:
        xenstat_uninit(handle)

if __name__ == "__main__": sys.exit(main(sys.argv))
